<?php

namespace CityBeach\CatchFeeder\Model;

use CityBeach\CatchFeeder\Api\OrderingInterface;
use CityBeach\CatchFeeder\Api\Data\OrderInterface;
use \Magento\Framework\ObjectManagerInterface;

class Ordering implements OrderingInterface
{
    /*
     * @var \Magento\Framework\ObjectManagerInterface $objectManager
     */
    private $objectManager;

    public function __construct(
        ObjectManagerInterface $objectManager
    )
    {
        $this->objectManager = $objectManager;
    }

    /**
     * Create an order.
     *
     * @api
     * @param OrderInterface $order
     * @return string
     * @throws \Exception on a validation error or order processing failure.
     */
    public function execute(OrderInterface $customOrder) {
        $status = 'success';
        $message = '';
        $cart_order_id = null;

        try {
            /** @var $magentoQuoteBuilder \CityBeach\CatchFeeder\Model\OrderBuilder */
            $builder = $this->objectManager->create('\CityBeach\CatchFeeder\Model\OrderBuilder', ['customOrder' => $customOrder]);
            $builder->execute();
            $order = $builder->getOrder();
            $cart_order_id = $order->getId();
            $message = $builder->getMessage();
        } catch (\Exception $exception) {
            $status = 'failure';
            $message = $exception->getMessage();
            $cart_order_id = null;
        }
        return array(
            'result' => array(
                'status' => $status,
                'message' => $message,
                'orderId' => $cart_order_id,
            )
        );
    }
}
