<?php

namespace CityBeach\CatchFeeder\Model;

use CityBeach\CatchFeeder\Api\ProductInventoryInterface;
use Magento\Catalog\Api\Data\ProductInterface;

class ProductInventory implements ProductInventoryInterface
{
    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var \Magento\CatalogInventory\Model\Stock\StockItemRepository
     */
    private $stockItemRepository;

    /**
     * @var \Magento\Catalog\Helper\Product
     */
    private $productHelper;

    /**
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     * @param \Magento\CatalogInventory\Model\Stock\StockItemRepository $stockItemRepository
     * @param \Magento\Catalog\Helper\Product $productHelper
     */
    public function __construct(
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\CatalogInventory\Model\Stock\StockItemRepository $stockItemRepository,
        \Magento\Catalog\Helper\Product $productHelper
    ) {
        $this->productRepository = $productRepository;
        $this->stockItemRepository = $stockItemRepository;
        $this->productHelper = $productHelper;
    }

    /**
     * Load Block data collection by given search criteria
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param \Magento\Framework\Api\SearchCriteriaInterface $criteria
     * @return mixed
     */

    public function getList(\Magento\Framework\Api\SearchCriteriaInterface $criteria)
    {
        $products = $this->productRepository->getList($criteria);
        $productInventories = array();

        /* @var $product ProductInterface */
        foreach ($products->getItems() as $product) {
            $id = $product->getId();
            $productInventory = array(
                'id' => $id,
                'sku' => $product->getSku(),
            );
            $stock = $this->queryStock($product);
            $productInventory['stock'] = $stock;

            $productInventories[$id] = $productInventory;
        }

        return $productInventories;
    }


    private function queryStock($product) {
      $productStock = $this->stockItemRepository->get($product->getId());
      return $productStock->getQty();
    }

}
