<?php

namespace CityBeach\CatchFeeder\Model;

use CityBeach\CatchFeeder\Api\ProductSummaryRepositoryInterface;
use Magento\Catalog\Api\Data\ProductInterface;

class ProductSummaryRepository implements ProductSummaryRepositoryInterface
{
    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var \Magento\CatalogInventory\Model\Stock\StockItemRepository
     */
    private $stockItemRepository;

    /**
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     * @param \Magento\CatalogInventory\Model\Stock\StockItemRepository $stockItemRepository
     */
    public function __construct(
      \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
      \Magento\CatalogInventory\Model\Stock\StockItemRepository $stockItemRepository)
    {
      $this->productRepository = $productRepository;
      $this->stockItemRepository = $stockItemRepository;
    }

    /**
     * Load Block data collection by given search criteria
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param \Magento\Framework\Api\SearchCriteriaInterface $criteria
     * @return Array
     */

    public function getList(\Magento\Framework\Api\SearchCriteriaInterface $criteria)
    {
        $products = $this->productRepository->getList($criteria);
        $productSummaries = array();


        /* @var $product ProductInterface */
        foreach ($products->getItems() as $product) {
            $id = $product->getId();
            $typeInstance = $product->getTypeInstance();
            $productSummary = array(
                'id' => $id,
                'sku' => $product->getSku(),
                'name' => $product->getName(),
                'price' => $product->getPrice(),
                'weight' => $product->getWeight(),
                'status' => $product->getStatus(),
                'visibility' => $product->getVisibility(),
                'type' => $product->getTypeId(),
                'created' => $product->getCreatedAt(),
                'updated' => $product->getUpdatedAt(),
            );
            $attributes = array();
            foreach ($product->getData() as $key => $value) {
                if (!empty($value) || strpos($key, '_') !== 0) {
                    $attributes[$key] = $value;
                }
            }
            $productSummary['attributes'] = $attributes;
            if ($product->getTypeId() === 'configurable') {
                $used = $typeInstance->getUsedProducts($product);
                $related = array();
                foreach ($used as $use) {
                    $related[] = $use->getId();
                }
                $productSummary['related'] = $related;
            }
            elseif ($product->getTypeId() === 'grouped') {
              $associatedIds = $typeInstance->getAssociatedProductIds($product);
              $related = array();
              foreach ($associatedIds as $associatedId) {
                $related[] = $associatedId;
              }
              $productSummary['related'] = $related;
            }

          $stock = $this->queryStock($product);
            $productSummary['stock'] = $stock;
            $productSummaries[$id] = $productSummary;
        }

        return $productSummaries;
    }

    private function queryStock($product) {
      $productStock = $this->stockItem->get($product->getId());
      return $productStock->getQty();
    }
}
